#include "boot_sectors.h"
#include "simple_encryption.h"
#include "unique_id.h"
#include "stm32f4xx_flash.h"

#include "stm32f4xx_hal.h"
#include "stm32f4xx_hal_crc.h"
#include "stm32f4xx_hal_gpio.h"
#include "stm32f4xx_hal_iwdg.h"


typedef struct
{
    GPIO_TypeDef * gpio;
    uint16_t       pins;
} LED;


#ifdef DEV_BOARD
LED led = { GPIOC, GPIO_PIN_7 };
const GPIO_InitTypeDef ledInit = { GPIO_PIN_7, GPIO_MODE_OUTPUT_PP, GPIO_PULLUP, GPIO_SPEED_FAST, 0 };
#define GPIO_CLK_ENABLE     __GPIOC_CLK_ENABLE
#else
LED led = { GPIOG, GPIO_PIN_11 };
const GPIO_InitTypeDef ledInit =  { GPIO_PIN_11, GPIO_MODE_OUTPUT_PP, GPIO_NOPULL, GPIO_SPEED_LOW, 0 };
#define GPIO_CLK_ENABLE     __GPIOG_CLK_ENABLE
#endif


#define SRAM_SIZE           (128*1024)

#define PROG_DESTINATION    (SRAM_BASE)
#define PROG_MAX_SIZE       (SRAM_SIZE)


/// Hand tuned millisecond delay.
void delay (unsigned int ms)
{
    while (ms--)
    {
        for (unsigned int i = 0; i < 1260; ++i)
            ;
    }
}


void flash (unsigned int ms)
{
    led.gpio->ODR = led.pins;
    delay(ms);
    led.gpio->ODR &= ~led.pins;
}


/// Error handler, and default ISR.
void error_handler()
{
    while (1)
    {
        IWDG->KR = KR_KEY_RELOAD;
        flash(100);
        delay(200);
        IWDG->KR = KR_KEY_RELOAD;
        flash(200);
        delay(100);
    }
}


void copy_program (size_t sector)
{
    uint32_t key = munged_id();
    const uint32_t * data = (uint32_t *)flashSectors[sector].address;

    uint32_t size = *data++;
    uint32_t crc  = *data++;

    CRC->CR = CRC_CR_RESET;
    uint32_t * prog = (uint32_t *)PROG_DESTINATION;

    if (size & 0x80000000)  // signals that this uses simple encryption
    {
        size = ~size;
        if (size > PROG_MAX_SIZE)
        {
            error_handler();
        }

        for (uint32_t i = 0; i < size; i += sizeof (uint32_t))
        {
            uint32_t x = *data++;
            simple_decrypt(&x, sizeof x);
            *prog++ = x;
            CRC->DR = x;
        }
    }
    else    // "normal" encryption using the ARM's UID.
    {
        size ^= key;
        crc  ^= key;

        if (size > PROG_MAX_SIZE)
        {
            error_handler();
        }

        for (uint32_t i = 0; i < size; i += sizeof (uint32_t))
        {
            const uint32_t x = *data++ ^ key;
            key = key << 13 | key >> (32-13);
            *prog++ = x;
            CRC->DR = x;
        }
    }

    key = 0;   // don't leave key in RAM for all to see.

    if (CRC->DR != crc)
    {
        error_handler();
    }
}


int main (void)
{
    // no need to call HAL_Init() or initialise_systemClock() - basic default
    // system initialisation is sufficient.

    __CRC_CLK_ENABLE();
    GPIO_CLK_ENABLE();
    HAL_GPIO_Init(led.gpio, (GPIO_InitTypeDef*)&ledInit);

    flash(50);

    const size_t sector = find_boot_sector();

    copy_program (sector);

    // Stop IWDG when debugger halts CPU
    __HAL_FREEZE_IWDG_DBGMCU();

    // Start the watchdog : leave the default setting so we get ~500ms timeout
    IWDG->KR = KR_KEY_ENABLE;

    // Reset vector. vector 0 is the initial stack pointer.
    // Refer to Cortex M4 programming manual, 2.3.4:
    // http://www.st.com/web/en/resource/technical/document/programming_manual/DM00046982.pdf
    void (*reset)() = (void(*)())(((uint32_t *)PROG_DESTINATION)[1]);

    // Only jump if its in range.
    // This is a very basic check of validity, but something more restrictive
    // might prevent a different future boot arrangement.
    if (((uint32_t)reset >= PROG_DESTINATION) && ((uint32_t)reset < (PROG_DESTINATION + PROG_MAX_SIZE)))
    {
        reset();
    }

    error_handler();
}
